@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,18 +21,18 @@
 
 /** Setup user password
  *
- * @param h		VE handler.
- * @param veid		VE id.
+ * @param h		CT handler.
+ * @param veid		CT ID.
  * @param actions	distribution action scripts.
- * @param root		VE root.
+ * @param root		CT root.
  * @param pw		list of user:password parameters.
  * @return		0 on success.
  */
 int vps_pw_configure(vps_handler *h, envid_t veid, dist_actions *actions,
-	char *root, list_head_t *pw);
+	const char *root, list_head_t *pw);
 int vps_configure(vps_handler *h, envid_t veid, dist_actions *actions,
-	char *root, int op, vps_param *param, int state);
+	const char *root, int op, vps_param *param, int state);
 int vps_ip_configure(vps_handler *h, envid_t veid, dist_actions *actions,
-	char *root, int op, net_param *net, int state);
+	const char *root, int op, net_param *net, int state);
 const char *state2str(int state);
 #endif
