@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2000-2010, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -52,9 +52,10 @@ static char *e_buf = g_outbuffer + sizeof(g_outbuffer) - 1;
 static char *host_pattern = NULL;
 static char *name_pattern = NULL;
 static char *desc_pattern = NULL;
+static char *dumpdir = NULL;
 static int vzctlfd;
 static struct Cfield_order *g_field_order = NULL;
-struct Cfield_order *last_field = NULL;
+static int is_last_field;
 static char *default_field_order = "veid,numproc,status,ip,hostname";
 static char *default_nm_field_order = "veid,numproc,status,ip,name";
 static int g_sort_field = 0;
@@ -66,6 +67,7 @@ static int show_hdr = 1;
 static int all_ve = 0;
 static int only_stopped_ve = 0;
 static int with_names = 0;
+static long __clk_tck = -1;
 
 char logbuf[32];
 char *plogbuf = logbuf;
@@ -85,12 +87,47 @@ static inline int get_run_ve(int update)
 #endif
 
 
-static void print_hostname(struct Cveinfo *p, int index);
-static void print_name(struct Cveinfo *p, int index);
-static void print_ip(struct Cveinfo *p, int index);
-static void print_description(struct Cveinfo *p, int index);
-
 /* Print functions */
+#define PRINT_STR_FIELD(fieldname, length) \
+static void print_ ## fieldname(struct Cveinfo *p, int index) \
+{ \
+	int r; \
+	char *str = "-"; \
+ \
+	if (p->fieldname != NULL) \
+		str = p->fieldname; \
+	r = snprintf(p_outbuffer, e_buf - p_outbuffer, \
+		"%-" #length "s", str); \
+	if (!is_last_field) \
+		r = length; \
+	p_outbuffer += r; \
+}
+
+PRINT_STR_FIELD(hostname, 32)
+PRINT_STR_FIELD(name, 32)
+PRINT_STR_FIELD(description, 32)
+PRINT_STR_FIELD(ostemplate, 32)
+
+static void print_ip(struct Cveinfo *p, int index)
+{
+	int r;
+	char *str = "-";
+	char *ch;
+
+	if (p->ip != NULL)
+		str = p->ip;
+	if (!is_last_field)
+	{
+		/* Fixme: dont destroy original string */
+		if ((ch = strchr(str, ' ')) != NULL)
+			*ch = 0;
+	}
+	r = snprintf(p_outbuffer, e_buf - p_outbuffer, "%-15s", str);
+	if (!is_last_field)
+		r = 15;
+	p_outbuffer += r;
+}
+
 static void print_veid(struct Cveinfo *p, int index)
 {
 	p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%10d", p->veid);
@@ -98,16 +135,40 @@ static void print_veid(struct Cveinfo *p, int index)
 
 static void print_status(struct Cveinfo *p, int index)
 {
-	p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%-7s", ve_status[p->status]);
+	p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%-9s", ve_status[p->status]);
 }
 
 static void print_laverage(struct Cveinfo *p, int index)
 {
-	if (p->la == NULL)
+	if (p->cpustat == NULL)
 		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%14s", "-");
 	else
 		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%1.2f/%1.2f/%1.2f",
-			p->la->la[0], p->la->la[1], p->la->la[2]);
+			p->cpustat->la[0], p->cpustat->la[1], p->cpustat->la[2]);
+}
+
+static void print_uptime(struct Cveinfo *p, int index)
+{
+	if (p->cpustat == NULL)
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer,
+				"%15s", "-");
+	else
+	{
+		unsigned int days, hours, min, secs;
+
+		days  = (unsigned int)(p->cpustat->uptime / (60 * 60 * 24));
+		min = (unsigned int)(p->cpustat->uptime / 60);
+		hours = min / 60;
+		hours = hours % 24;
+		min = min % 60;
+		secs = (unsigned int)(p->cpustat->uptime -
+				(60ull * min + 60ull * 60 * hours +
+				 60ull * 60 * 24 * days));
+
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer,
+				"%.3dd%.2dh:%.2dm:%.2ds",
+				days, hours, min, secs);
+	}
 }
 
 static void print_cpulimit(struct Cveinfo *p, int index)
@@ -119,52 +180,78 @@ static void print_cpulimit(struct Cveinfo *p, int index)
 			p->cpu->limit[index]);
 }
 
-#define PRINT_UL_RES(fn, res, name)					\
-static void fn(struct Cveinfo *p, int index)		  		\
+static void print_ioprio(struct Cveinfo *p, int index)
+{
+	if (p->io.ioprio < 0)
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%3s", "-");
+	else
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%3d",
+			p->io.ioprio);
+}
+
+static void print_onboot(struct Cveinfo *p, int index)
+{
+	p_outbuffer += snprintf(p_outbuffer, e_buf-p_outbuffer,
+			"%6s", p->onboot == YES ? "yes" : "no");
+}
+
+static void print_bootorder(struct Cveinfo *p, int index)
+{
+	if (p->bootorder == NULL)
+		p_outbuffer += snprintf(p_outbuffer, e_buf-p_outbuffer,
+				"%10s", "-");
+	else
+		p_outbuffer += snprintf(p_outbuffer, e_buf-p_outbuffer,
+				"%10lu", p->bootorder[index]);
+}
+
+#define PRINT_UBC(name)							\
+static void print_ubc_ ## name(struct Cveinfo *p, int index)		\
 {									\
-	if (p->res == NULL ||						\
+	if (p->ubc == NULL ||						\
 		(p->status != VE_RUNNING &&				\
 			(index == 0 || index == 1 || index == 4)))	\
 		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%10s", "-");	\
 	else								\
 		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%10lu",		\
-					p->res->name[index]);		\
+					p->ubc->name[index]);		\
 }									\
 
-PRINT_UL_RES(print_ubc_kmemsize, ubc, kmemsize)
-PRINT_UL_RES(print_ubc_lockedpages, ubc, lockedpages)
-PRINT_UL_RES(print_ubc_privvmpages, ubc, privvmpages)
-PRINT_UL_RES(print_ubc_shmpages, ubc, shmpages)
-PRINT_UL_RES(print_ubc_numproc, ubc, numproc)
-PRINT_UL_RES(print_ubc_physpages, ubc, physpages)
-PRINT_UL_RES(print_ubc_vmguarpages, ubc, vmguarpages)
-PRINT_UL_RES(print_ubc_oomguarpages, ubc, oomguarpages)
-PRINT_UL_RES(print_ubc_numtcpsock, ubc, numtcpsock)
-PRINT_UL_RES(print_ubc_numflock, ubc, numflock)
-PRINT_UL_RES(print_ubc_numpty, ubc, numpty)
-PRINT_UL_RES(print_ubc_numsiginfo, ubc, numsiginfo)
-PRINT_UL_RES(print_ubc_tcpsndbuf, ubc, tcpsndbuf)
-PRINT_UL_RES(print_ubc_tcprcvbuf, ubc, tcprcvbuf)
-PRINT_UL_RES(print_ubc_othersockbuf, ubc, othersockbuf)
-PRINT_UL_RES(print_ubc_dgramrcvbuf, ubc, dgramrcvbuf)
-PRINT_UL_RES(print_ubc_numothersock, ubc, numothersock)
-PRINT_UL_RES(print_ubc_dcachesize, ubc, dcachesize)
-PRINT_UL_RES(print_ubc_numfile, ubc, numfile)
-PRINT_UL_RES(print_ubc_numiptent, ubc, numiptent)
+PRINT_UBC(kmemsize)
+PRINT_UBC(lockedpages)
+PRINT_UBC(privvmpages)
+PRINT_UBC(shmpages)
+PRINT_UBC(numproc)
+PRINT_UBC(physpages)
+PRINT_UBC(vmguarpages)
+PRINT_UBC(oomguarpages)
+PRINT_UBC(numtcpsock)
+PRINT_UBC(numflock)
+PRINT_UBC(numpty)
+PRINT_UBC(numsiginfo)
+PRINT_UBC(tcpsndbuf)
+PRINT_UBC(tcprcvbuf)
+PRINT_UBC(othersockbuf)
+PRINT_UBC(dgramrcvbuf)
+PRINT_UBC(numothersock)
+PRINT_UBC(dcachesize)
+PRINT_UBC(numfile)
+PRINT_UBC(numiptent)
+PRINT_UBC(swappages)
 
-#define PRINT_DQ_RES(fn, res, name)					\
-static void fn(struct Cveinfo *p, int index)		  		\
+#define PRINT_DQ(name)							\
+static void print_ ## name(struct Cveinfo *p, int index)		\
 {									\
-	if (p->res == NULL ||						\
+	if (p->quota == NULL ||						\
 		(p->status != VE_RUNNING && (index == 0)))		\
 		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%10s", "-");	\
 	else								\
 		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%10lu",		\
-					p->res->name[index]);		\
+					p->quota->name[index]);		\
 }									\
 
-PRINT_DQ_RES(print_dq_blocks, quota, diskspace)
-PRINT_DQ_RES(print_dq_inodes, quota, diskinodes)
+PRINT_DQ(diskspace)
+PRINT_DQ(diskinodes)
 
 /* Sort functions */
 
@@ -186,19 +273,30 @@ int none_sort_fn(const void *val1, const void *val2)
 
 int laverage_sort_fn(const void *val1, const void *val2)
 {
-	const struct Cla *la1 = ((const struct Cveinfo *)val1)->la;
-	const struct Cla *la2 = ((const struct Cveinfo *)val2)->la;
+	const struct Ccpustat *st1 = ((const struct Cveinfo *)val1)->cpustat;
+	const struct Ccpustat *st2 = ((const struct Cveinfo *)val2)->cpustat;
 	int res;
 
-	if ((res = check_empty_param(la1, la2)) == 2)
+	if ((res = check_empty_param(st1, st2)) == 2)
 		return res;
-	res = (la1->la[0] - la2->la[0]) * 100;
+	res = (st1->la[0] - st2->la[0]) * 100;
 	if (res != 0)
 		return res;
-	res = (la1->la[1] - la2->la[1]) * 100;
+	res = (st1->la[1] - st2->la[1]) * 100;
 	if (res != 0)
 		return res;
-	return (la1->la[2] - la2->la[2]) * 100;
+	return (st1->la[2] - st2->la[2]) * 100;
+}
+
+int uptime_sort_fn(const void *val1, const void *val2)
+{
+	struct Ccpustat *st1 = ((const struct Cveinfo *)val1)->cpustat;
+	struct Ccpustat *st2 = ((const struct Cveinfo *)val2)->cpustat;
+	int res;
+
+	if ((res = check_empty_param(st1, st2)) != 2)
+		return res;
+	return (st2->uptime - st1->uptime);
 }
 
 int id_sort_fn(const void *val1, const void *val2)
@@ -219,8 +317,36 @@ int status_sort_fn(const void *val1, const void *val2)
 	return res;
 }
 
-#define SORT_STR_FN(fn, name)	 					\
-int fn(const void *val1, const void *val2)				\
+int bootorder_sort_fn(const void *val1, const void *val2)
+{
+	int ret;
+	unsigned long *r1 = ((const struct Cveinfo*)val1)->bootorder;
+	unsigned long *r2 = ((const struct Cveinfo*)val2)->bootorder;
+
+	ret = check_empty_param(r1, r2);
+	switch (ret) {
+		case 0: /* both NULL */
+			return !id_sort_fn(val1, val2);
+		case 2: /* both not NULL */
+			break;
+		default: /* one is NULL, other is not */
+			return ret;
+	}
+
+	if (*r1 == *r2)
+		return !id_sort_fn(val1, val2);
+
+	return (*r1 > *r2);
+}
+
+int ioprio_sort_fn(const void *val1, const void *val2)
+{
+	return ((const struct Cveinfo *)val1)->io.ioprio >
+		((const struct Cveinfo *)val2)->io.ioprio;
+}
+
+#define SORT_STR_FN(name)						\
+int name ## _sort_fn(const void *val1, const void *val2)		\
 {									\
 	const char *h1 = ((const struct Cveinfo*)val1)->name;		\
 	const char *h2 = ((const struct Cveinfo*)val2)->name;		\
@@ -230,167 +356,78 @@ int fn(const void *val1, const void *val2)				\
 	return ret;					\
 }
 
-SORT_STR_FN(hostnm_sort_fn, hostname)
-SORT_STR_FN(name_sort_fn, name)
-SORT_STR_FN(description_sort_fn, description)
-SORT_STR_FN(ip_sort_fn, ip)
+SORT_STR_FN(hostname)
+SORT_STR_FN(name)
+SORT_STR_FN(description)
+SORT_STR_FN(ostemplate)
+SORT_STR_FN(ip)
 
-#define SORT_UL_RES(fn, type, res, name, index)				\
+#define SORT_UL_RES(fn, res, name, index)				\
 int fn(const void *val1, const void *val2)				\
 {									\
-	const struct type *r1 = ((const struct Cveinfo *)val1)->res;	\
-	const struct type *r2 = ((const struct Cveinfo *)val2)->res;	\
+	const struct C ## res *r1 = ((const struct Cveinfo *)val1)->res;\
+	const struct C ## res *r2 = ((const struct Cveinfo *)val2)->res;\
 	int ret;							\
 	if ((ret = check_empty_param(r1, r2)) == 2)			\
 		ret = r1->name[index] > r2->name[index];		\
 	return ret;					\
 }
 
-SORT_UL_RES(kmemsize_h_sort_fn, Cubc, ubc, kmemsize, 0)
-SORT_UL_RES(kmemsize_m_sort_fn, Cubc, ubc, kmemsize, 1)
-SORT_UL_RES(kmemsize_l_sort_fn, Cubc, ubc, kmemsize, 2)
-SORT_UL_RES(kmemsize_b_sort_fn, Cubc, ubc, kmemsize, 3)
-SORT_UL_RES(kmemsize_f_sort_fn, Cubc, ubc, kmemsize, 4)
+#define SORT_UBC(res)							\
+SORT_UL_RES(res ## _h_sort_fn, ubc, res, 0)				\
+SORT_UL_RES(res ## _m_sort_fn, ubc, res, 1)				\
+SORT_UL_RES(res ## _l_sort_fn, ubc, res, 2)				\
+SORT_UL_RES(res ## _b_sort_fn, ubc, res, 3)				\
+SORT_UL_RES(res ## _f_sort_fn, ubc, res, 4)
 
-SORT_UL_RES(lockedpages_h_sort_fn, Cubc, ubc, lockedpages, 0)
-SORT_UL_RES(lockedpages_m_sort_fn, Cubc, ubc, lockedpages, 1)
-SORT_UL_RES(lockedpages_l_sort_fn, Cubc, ubc, lockedpages, 2)
-SORT_UL_RES(lockedpages_b_sort_fn, Cubc, ubc, lockedpages, 3)
-SORT_UL_RES(lockedpages_f_sort_fn, Cubc, ubc, lockedpages, 4)
+SORT_UBC(kmemsize)
+SORT_UBC(lockedpages)
+SORT_UBC(privvmpages)
+SORT_UBC(shmpages)
+SORT_UBC(numproc)
+SORT_UBC(physpages)
+SORT_UBC(vmguarpages)
+SORT_UBC(oomguarpages)
+SORT_UBC(numtcpsock)
+SORT_UBC(numflock)
+SORT_UBC(numpty)
+SORT_UBC(numsiginfo)
+SORT_UBC(tcpsndbuf)
+SORT_UBC(tcprcvbuf)
+SORT_UBC(othersockbuf)
+SORT_UBC(dgramrcvbuf)
+SORT_UBC(numothersock)
+SORT_UBC(dcachesize)
+SORT_UBC(numfile)
+SORT_UBC(numiptent)
+SORT_UBC(swappages)
 
-SORT_UL_RES(privvmpages_h_sort_fn, Cubc, ubc, privvmpages, 0)
-SORT_UL_RES(privvmpages_m_sort_fn, Cubc, ubc, privvmpages, 1)
-SORT_UL_RES(privvmpages_l_sort_fn, Cubc, ubc, privvmpages, 2)
-SORT_UL_RES(privvmpages_b_sort_fn, Cubc, ubc, privvmpages, 3)
-SORT_UL_RES(privvmpages_f_sort_fn, Cubc, ubc, privvmpages, 4)
+#define SORT_DQ(res)							\
+SORT_UL_RES(res ## _u_sort_fn, quota, res, 0)				\
+SORT_UL_RES(res ## _s_sort_fn, quota, res, 1)				\
+SORT_UL_RES(res ## _h_sort_fn, quota, res, 2)
 
-SORT_UL_RES(shmpages_h_sort_fn, Cubc, ubc, shmpages, 0)
-SORT_UL_RES(shmpages_m_sort_fn, Cubc, ubc, shmpages, 1)
-SORT_UL_RES(shmpages_l_sort_fn, Cubc, ubc, shmpages, 2)
-SORT_UL_RES(shmpages_b_sort_fn, Cubc, ubc, shmpages, 3)
-SORT_UL_RES(shmpages_f_sort_fn, Cubc, ubc, shmpages, 4)
+SORT_DQ(diskspace)
+SORT_DQ(diskinodes)
 
-SORT_UL_RES(numproc_h_sort_fn, Cubc, ubc, numproc, 0)
-SORT_UL_RES(numproc_m_sort_fn, Cubc, ubc, numproc, 1)
-SORT_UL_RES(numproc_l_sort_fn, Cubc, ubc, numproc, 2)
-SORT_UL_RES(numproc_b_sort_fn, Cubc, ubc, numproc, 3)
-SORT_UL_RES(numproc_f_sort_fn, Cubc, ubc, numproc, 4)
-
-SORT_UL_RES(physpages_h_sort_fn, Cubc, ubc, physpages, 0)
-SORT_UL_RES(physpages_m_sort_fn, Cubc, ubc, physpages, 1)
-SORT_UL_RES(physpages_l_sort_fn, Cubc, ubc, physpages, 2)
-SORT_UL_RES(physpages_b_sort_fn, Cubc, ubc, physpages, 3)
-SORT_UL_RES(physpages_f_sort_fn, Cubc, ubc, physpages, 4)
-
-SORT_UL_RES(vmguarpages_h_sort_fn, Cubc, ubc, vmguarpages, 0)
-SORT_UL_RES(vmguarpages_m_sort_fn, Cubc, ubc, vmguarpages, 1)
-SORT_UL_RES(vmguarpages_l_sort_fn, Cubc, ubc, vmguarpages, 2)
-SORT_UL_RES(vmguarpages_b_sort_fn, Cubc, ubc, vmguarpages, 3)
-SORT_UL_RES(vmguarpages_f_sort_fn, Cubc, ubc, vmguarpages, 4)
-
-SORT_UL_RES(oomguarpages_h_sort_fn, Cubc, ubc, oomguarpages, 0)
-SORT_UL_RES(oomguarpages_m_sort_fn, Cubc, ubc, oomguarpages, 1)
-SORT_UL_RES(oomguarpages_l_sort_fn, Cubc, ubc, oomguarpages, 2)
-SORT_UL_RES(oomguarpages_b_sort_fn, Cubc, ubc, oomguarpages, 3)
-SORT_UL_RES(oomguarpages_f_sort_fn, Cubc, ubc, oomguarpages, 4)
-
-SORT_UL_RES(numtcpsock_h_sort_fn, Cubc, ubc, numtcpsock, 0)
-SORT_UL_RES(numtcpsock_m_sort_fn, Cubc, ubc, numtcpsock, 1)
-SORT_UL_RES(numtcpsock_l_sort_fn, Cubc, ubc, numtcpsock, 2)
-SORT_UL_RES(numtcpsock_b_sort_fn, Cubc, ubc, numtcpsock, 3)
-SORT_UL_RES(numtcpsock_f_sort_fn, Cubc, ubc, numtcpsock, 4)
-
-SORT_UL_RES(numflock_h_sort_fn, Cubc, ubc, numflock, 0)
-SORT_UL_RES(numflock_m_sort_fn, Cubc, ubc, numflock, 1)
-SORT_UL_RES(numflock_l_sort_fn, Cubc, ubc, numflock, 2)
-SORT_UL_RES(numflock_b_sort_fn, Cubc, ubc, numflock, 3)
-SORT_UL_RES(numflock_f_sort_fn, Cubc, ubc, numflock, 4)
-
-SORT_UL_RES(numpty_h_sort_fn, Cubc, ubc, numpty, 0)
-SORT_UL_RES(numpty_m_sort_fn, Cubc, ubc, numpty, 1)
-SORT_UL_RES(numpty_l_sort_fn, Cubc, ubc, numpty, 2)
-SORT_UL_RES(numpty_b_sort_fn, Cubc, ubc, numpty, 3)
-SORT_UL_RES(numpty_f_sort_fn, Cubc, ubc, numpty, 4)
-
-SORT_UL_RES(numsiginfo_h_sort_fn, Cubc, ubc, numsiginfo, 0)
-SORT_UL_RES(numsiginfo_m_sort_fn, Cubc, ubc, numsiginfo, 1)
-SORT_UL_RES(numsiginfo_l_sort_fn, Cubc, ubc, numsiginfo, 2)
-SORT_UL_RES(numsiginfo_b_sort_fn, Cubc, ubc, numsiginfo, 3)
-SORT_UL_RES(numsiginfo_f_sort_fn, Cubc, ubc, numsiginfo, 4)
-
-SORT_UL_RES(tcpsndbuf_h_sort_fn, Cubc, ubc, tcpsndbuf, 0)
-SORT_UL_RES(tcpsndbuf_m_sort_fn, Cubc, ubc, tcpsndbuf, 1)
-SORT_UL_RES(tcpsndbuf_l_sort_fn, Cubc, ubc, tcpsndbuf, 2)
-SORT_UL_RES(tcpsndbuf_b_sort_fn, Cubc, ubc, tcpsndbuf, 3)
-SORT_UL_RES(tcpsndbuf_f_sort_fn, Cubc, ubc, tcpsndbuf, 4)
-
-SORT_UL_RES(tcprcvbuf_h_sort_fn, Cubc, ubc, tcprcvbuf, 0)
-SORT_UL_RES(tcprcvbuf_m_sort_fn, Cubc, ubc, tcprcvbuf, 1)
-SORT_UL_RES(tcprcvbuf_l_sort_fn, Cubc, ubc, tcprcvbuf, 2)
-SORT_UL_RES(tcprcvbuf_b_sort_fn, Cubc, ubc, tcprcvbuf, 3)
-SORT_UL_RES(tcprcvbuf_f_sort_fn, Cubc, ubc, tcprcvbuf, 4)
-
-SORT_UL_RES(othersockbuf_h_sort_fn, Cubc, ubc, othersockbuf, 0)
-SORT_UL_RES(othersockbuf_m_sort_fn, Cubc, ubc, othersockbuf, 1)
-SORT_UL_RES(othersockbuf_l_sort_fn, Cubc, ubc, othersockbuf, 2)
-SORT_UL_RES(othersockbuf_b_sort_fn, Cubc, ubc, othersockbuf, 3)
-SORT_UL_RES(othersockbuf_f_sort_fn, Cubc, ubc, othersockbuf, 4)
-
-SORT_UL_RES(dgramrcvbuf_h_sort_fn, Cubc, ubc, dgramrcvbuf, 0)
-SORT_UL_RES(dgramrcvbuf_m_sort_fn, Cubc, ubc, dgramrcvbuf, 1)
-SORT_UL_RES(dgramrcvbuf_l_sort_fn, Cubc, ubc, dgramrcvbuf, 2)
-SORT_UL_RES(dgramrcvbuf_b_sort_fn, Cubc, ubc, dgramrcvbuf, 3)
-SORT_UL_RES(dgramrcvbuf_f_sort_fn, Cubc, ubc, dgramrcvbuf, 4)
-
-SORT_UL_RES(numothersock_h_sort_fn, Cubc, ubc, numothersock, 0)
-SORT_UL_RES(numothersock_m_sort_fn, Cubc, ubc, numothersock, 1)
-SORT_UL_RES(numothersock_l_sort_fn, Cubc, ubc, numothersock, 2)
-SORT_UL_RES(numothersock_b_sort_fn, Cubc, ubc, numothersock, 3)
-SORT_UL_RES(numothersock_f_sort_fn, Cubc, ubc, numothersock, 4)
-
-SORT_UL_RES(dcachesize_h_sort_fn, Cubc, ubc, dcachesize, 0)
-SORT_UL_RES(dcachesize_m_sort_fn, Cubc, ubc, dcachesize, 1)
-SORT_UL_RES(dcachesize_l_sort_fn, Cubc, ubc, dcachesize, 2)
-SORT_UL_RES(dcachesize_b_sort_fn, Cubc, ubc, dcachesize, 3)
-SORT_UL_RES(dcachesize_f_sort_fn, Cubc, ubc, dcachesize, 4)
-
-SORT_UL_RES(numfile_h_sort_fn, Cubc, ubc, numfile, 0)
-SORT_UL_RES(numfile_m_sort_fn, Cubc, ubc, numfile, 1)
-SORT_UL_RES(numfile_l_sort_fn, Cubc, ubc, numfile, 2)
-SORT_UL_RES(numfile_b_sort_fn, Cubc, ubc, numfile, 3)
-SORT_UL_RES(numfile_f_sort_fn, Cubc, ubc, numfile, 4)
-
-SORT_UL_RES(numiptent_h_sort_fn, Cubc, ubc, numiptent, 0)
-SORT_UL_RES(numiptent_m_sort_fn, Cubc, ubc, numiptent, 1)
-SORT_UL_RES(numiptent_l_sort_fn, Cubc, ubc, numiptent, 2)
-SORT_UL_RES(numiptent_b_sort_fn, Cubc, ubc, numiptent, 3)
-SORT_UL_RES(numiptent_f_sort_fn, Cubc, ubc, numiptent, 4)
-
-SORT_UL_RES(dqblocks_u_sort_fn, Cquota, quota, diskspace, 0)
-SORT_UL_RES(dqblocks_s_sort_fn, Cquota, quota, diskspace, 1)
-SORT_UL_RES(dqblocks_h_sort_fn, Cquota, quota, diskspace, 2)
-
-SORT_UL_RES(dqinodes_u_sort_fn, Cquota, quota, diskinodes, 0)
-SORT_UL_RES(dqinodes_s_sort_fn, Cquota, quota, diskinodes, 1)
-SORT_UL_RES(dqinodes_h_sort_fn, Cquota, quota, diskinodes, 2)
-
-SORT_UL_RES(cpulimit_sort_fn, Ccpu, cpu, limit, 0)
-SORT_UL_RES(cpuunits_sort_fn, Ccpu, cpu, limit, 1)
+SORT_UL_RES(cpulimit_sort_fn, cpu, limit, 0)
+SORT_UL_RES(cpuunits_sort_fn, cpu, limit, 1)
 
 struct Cfield field_names[] =
 {
 /* ctid should have index 0 */
 {"ctid", "CTID", "%10s", 0, RES_NONE, print_veid, id_sort_fn},
 /* veid is for backward compatibility -- will be removed later */
-{"veid", "CTID", "%10s", 0,  RES_NONE, print_veid, id_sort_fn},
+{"veid", "CTID", "%10s", 0, RES_NONE, print_veid, id_sort_fn},
 /* vpsid is for backward compatibility -- will be removed later */
-{"vpsid" , "CTID", "%10s",  0, RES_NONE, print_veid, id_sort_fn},
+{"vpsid", "CTID", "%10s", 0, RES_NONE, print_veid, id_sort_fn},
 
-{"hostname", "HOSTNAME", "%-32s", 0, RES_HOSTNAME, print_hostname, hostnm_sort_fn},
-{"name", "NAME", "%-32s", 0, RES_NAME, print_name, name_sort_fn},
-{"description", "DESCRIPTION", "%-32s", 0, RES_DESCRIPTION, print_description, description_sort_fn }, 
+{"hostname", "HOSTNAME", "%-32s", 0, RES_HOSTNAME, print_hostname, hostname_sort_fn},
+{"name", "NAME", "%-32s", 0, RES_NONE, print_name, name_sort_fn},
+{"description", "DESCRIPTION", "%-32s", 0, RES_NONE, print_description, description_sort_fn },
+{"ostemplate", "OSTEMPLATE", "%-32s", 0, RES_NONE, print_ostemplate, ostemplate_sort_fn },
 {"ip", "IP_ADDR", "%-15s", 0, RES_IP, print_ip, ip_sort_fn},
-{"status", "STATUS", "%-7s", 0, RES_NONE, print_status, status_sort_fn},
+{"status", "STATUS", "%-9s", 0, RES_NONE, print_status, status_sort_fn},
 /*	UBC	*/
 {"kmemsize", "KMEMSIZE", "%10s", 0, RES_UBC, print_ubc_kmemsize, kmemsize_h_sort_fn},
 {"kmemsize.m", "KMEMSIZE.M", "%10s", 1, RES_UBC, print_ubc_kmemsize, kmemsize_m_sort_fn},
@@ -511,93 +548,33 @@ struct Cfield field_names[] =
 {"numiptent.l", "NIPTENT.L", "%10s", 3, RES_UBC,print_ubc_numiptent, numiptent_l_sort_fn},
 {"numiptent.f", "NIPTENT.F", "%10s", 4, RES_UBC,print_ubc_numiptent, numiptent_f_sort_fn},
 
-{"diskspace", "DQBLOCKS", "%10s", 0, RES_QUOTA, print_dq_blocks, dqblocks_u_sort_fn},
-{"diskspace.s", "DQBLOCKS.S", "%10s", 1, RES_QUOTA, print_dq_blocks, dqblocks_s_sort_fn},
-{"diskspace.h", "DQBLOCKS.H", "%10s", 2, RES_QUOTA, print_dq_blocks, dqblocks_h_sort_fn},
+{"swappages", "SWAPP", "%10s", 0, RES_UBC,print_ubc_swappages, swappages_h_sort_fn},
+{"swappages.m", "SWAPP.M", "%10s", 1, RES_UBC,print_ubc_swappages, swappages_m_sort_fn},
+{"swappages.b", "SWAPP.B", "%10s", 2, RES_UBC,print_ubc_swappages, swappages_b_sort_fn},
+{"swappages.l", "SWAPP.L", "%10s", 3, RES_UBC,print_ubc_swappages, swappages_l_sort_fn},
+{"swappages.f", "SWAPP.F", "%10s", 4, RES_UBC,print_ubc_swappages, swappages_f_sort_fn},
 
-{"diskinodes", "DQINODES", "%10s", 0, RES_QUOTA, print_dq_inodes, dqinodes_u_sort_fn},
-{"diskinodes.s", "DQINODES.S", "%10s", 1, RES_QUOTA, print_dq_inodes, dqinodes_u_sort_fn},
-{"diskinodes.h", "DQINODES.H", "%10s", 2, RES_QUOTA, print_dq_inodes, dqinodes_u_sort_fn},
+{"diskspace", "DSPACE", "%10s", 0, RES_QUOTA, print_diskspace, diskspace_u_sort_fn},
+{"diskspace.s", "DSPACE.S", "%10s", 1, RES_QUOTA, print_diskspace, diskspace_s_sort_fn},
+{"diskspace.h", "DSPACE.H", "%10s", 2, RES_QUOTA, print_diskspace, diskspace_h_sort_fn},
 
-{"laverage", "LAVERAGE", "%14s", 0, RES_LA, print_laverage, laverage_sort_fn},
+{"diskinodes", "DINODES", "%10s", 0, RES_QUOTA, print_diskinodes, diskinodes_u_sort_fn},
+{"diskinodes.s", "DINODES.S", "%10s", 1, RES_QUOTA, print_diskinodes, diskinodes_u_sort_fn},
+{"diskinodes.h", "DINODES.H", "%10s", 2, RES_QUOTA, print_diskinodes, diskinodes_u_sort_fn},
+
+{"laverage", "LAVERAGE", "%14s", 0, RES_CPUSTAT, print_laverage, laverage_sort_fn},
+{"uptime", "UPTIME", "%15s", 1, RES_CPUSTAT, print_uptime, uptime_sort_fn},
 
 {"cpulimit", "CPULIM", "%7s", 0, RES_CPU, print_cpulimit, cpulimit_sort_fn},
-{"cpuunits", "CPUUNI", "%7s", 1, RES_CPU, print_cpulimit, cpuunits_sort_fn}
+{"cpuunits", "CPUUNI", "%7s", 1, RES_CPU, print_cpulimit, cpuunits_sort_fn},
+
+{"ioprio", "IOP", "%3s", 1, RES_NONE, print_ioprio, ioprio_sort_fn},
+
+{"onboot", "ONBOOT", "%6s", 0, RES_NONE, print_onboot, none_sort_fn},
+{"bootorder", "BOOTORDER", "%10s", 0, RES_NONE,
+	print_bootorder, bootorder_sort_fn},
 };
 
-static void print_hostname(struct Cveinfo *p, int index)
-{
-	int r;
-	char *str = "-";
-
-	if (p->hostname != NULL)
-		str = p->hostname;
-	r = snprintf(p_outbuffer, e_buf - p_outbuffer, "%-32s", str);
-	if (last_field != NULL &&
-		field_names[last_field->order].res_type != RES_HOSTNAME)
-	{
-		r = 32;
-	}
-	p_outbuffer +=  r;
-}
-
-static void print_name(struct Cveinfo *p, int index)
-{
-	int r;
-	char *str = "-";
-
-	if (p->name != NULL)
-		str = p->name;
-	r = snprintf(p_outbuffer, e_buf - p_outbuffer, "%-32s", str);
-	if (last_field != NULL &&
-		field_names[last_field->order].res_type != RES_NAME)
-	{
-		r = 32;
-	}
-	p_outbuffer +=  r;
-}
-
-static void print_description(struct Cveinfo *p, int index)
-{
-	int r;
-	char *str = "-";
-
-	if (p->description != NULL)
-		str = p->description;
-
-	r = snprintf(p_outbuffer, e_buf - p_outbuffer, "%-32s", str);
-	if (last_field != NULL &&
-		field_names[last_field->order].res_type != RES_DESCRIPTION)
-	{
-		r = 32;
-	}
-	p_outbuffer += r;
-}
-
-static void print_ip(struct Cveinfo *p, int index)
-{
-	int r;
-	char *str = "-";
-	char *ch;
-
-	if (p->ip != NULL)
-		str = p->ip;
-	if (last_field != NULL &&
-		field_names[last_field->order].res_type != RES_IP)
-	{
-		/* Fixme: dont destroy original string */
-		if ((ch = strchr(str, ' ')) != NULL)
-			*ch = 0;
-	}
-	r = snprintf(p_outbuffer, e_buf - p_outbuffer, "%-15s", str);
-	if (last_field != NULL &&
-		field_names[last_field->order].res_type != RES_IP)
-	{
-		r = 15;
-	}
-	p_outbuffer +=  r;
-}
-
 void *x_malloc(int size)
 {
 	void *p;
@@ -621,21 +598,24 @@ void *x_realloc(void *ptr, int size)
 
 void usage()
 {
-	fprintf(stderr, "Usage: vzlist [-a] [-o name[,name...]] [-s {name|-name}] [-h <pattern>] [-N <pattern>]\n");
-	fprintf(stderr, "\t\t[-H] [-S] [veid [veid ...]|-1]\n");
-	fprintf(stderr, "	vzlist -L\n\n");
-	fprintf(stderr, "\t--all -a\t list of all containers\n");
-	fprintf(stderr, "\t--output -o\t output only specified parameters\n");
-	fprintf(stderr, "\t--hostname -h\t hostname search pattern\n");
-	fprintf(stderr, "\t--name -n\t display container's name\n");
-	fprintf(stderr, "\t--name_filter -N\t name search patter\n");
-	fprintf(stderr, "\t--description -d\t description search pattern\n");
-	fprintf(stderr, "\t--sort -s\t sort by specified parameter, - sign before parametr\n");
-	fprintf(stderr, "\t\t\t mean sort in reverse order\n");
-	fprintf(stderr, "\t--no-header -H\t supress display header\n");
-	fprintf(stderr, "\t--stopped -S\t list of stopped Ve\n");
-	fprintf(stderr, "\t--list -L\t list of allowed parameters\n");
-	return;
+	fprintf(stderr, "\n"
+"Usage:	vzlist	[-a|-S] [-n] [-H] [-o field[,field...]] [-s [-]field]\n"
+"		[-h pattern] [-N pattern] [-d pattern] [CTID [CTID ...]|-1]\n"
+"	vzlist -L\n"
+"\n"
+"Options:\n"
+"	-a, --all		list all containers\n"
+"	-S, --stopped		list stopped containers\n"
+"	-n, --name		display containers' names\n"
+"	-H, --no-header		suppress columns header\n"
+"	-o, --output		output only specified fields\n"
+"	-s, --sort		sort by the specified field\n"
+"				('-field' to reverse sort order)\n"
+"	-h, --hostname		filter CTs by hostname pattern\n"
+"	-N, --name_filter	filter CTs by name pattern\n"
+"	-d, --description	filter CTs by description pattern\n"
+"	-L, --list		get possible field names\n"
+	);
 }
 
 int id_search_fn(const void* val1, const void* val2)
@@ -648,6 +628,17 @@ int veid_search_fn(const void* val1, const void* val2)
 	return (*(const int *)val1 - *(const int *)val2);
 }
 
+char* trim_eol_space(char *sp, char *ep)
+{
+/*	if (ep == NULL)
+		ep = sp + strlen(sp); */
+
+	ep--;
+	while (isspace(*ep) && ep >= sp) *ep-- = '\0';
+
+	return sp;
+}
+
 void print_hdr()
 {
 	struct Cfield_order *p;
@@ -655,16 +646,21 @@ void print_hdr()
 
 	for (p = g_field_order; p != NULL; p = p->next) {
 		f = p->order;
-		printf(field_names[f].hdr_fmt, field_names[f].hdr);
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer,
+				field_names[f].hdr_fmt, field_names[f].hdr);
+		if (p_outbuffer >= e_buf)
+			break;
 		if (p->next != NULL)
-			printf(" ");
+			*p_outbuffer++ = ' ';
 	}
-	printf("\n");
+	printf("%s\n", trim_eol_space(g_outbuffer, p_outbuffer));
+	g_outbuffer[0] = 0;
+	p_outbuffer = g_outbuffer;
 }
 
 /*
-   1 - match
-   0 - do not match
+	1 - match
+	0 - do not match
 */
 inline int check_pattern(char *str, char *pat)
 {
@@ -675,7 +671,7 @@ inline int check_pattern(char *str, char *pat)
 	return !fnmatch(pat, str, 0);
 }
 
-int filter_by_hostname()
+void filter_by_hostname()
 {
 	int i;
 
@@ -683,10 +679,9 @@ int filter_by_hostname()
 		if (!check_pattern(veinfo[i].hostname, host_pattern))
 			veinfo[i].hide = 1;
 	}
-	return 0;
 }
 
-int filter_by_name()
+void filter_by_name()
 {
 	int i;
 
@@ -694,10 +689,9 @@ int filter_by_name()
 		if (!check_pattern(veinfo[i].name, name_pattern))
 			veinfo[i].hide = 1;
 	}
-	return 0;
 }
 
-int filter_by_description()
+void filter_by_description()
 {
 	int i;
 
@@ -705,7 +699,6 @@ int filter_by_description()
 		if (!check_pattern(veinfo[i].description, desc_pattern))
 			veinfo[i].hide = 1;
 	}
-	return 0;
 }
 
 void print_ve()
@@ -729,8 +722,11 @@ void print_ve()
 			continue;
 		if (only_stopped_ve && veinfo[idx].status == VE_RUNNING)
 			continue;
+		is_last_field = 0;
 		for (p = g_field_order; p != NULL; p = p->next) {
 			f = p->order;
+			if (p->next == NULL)
+				is_last_field = 1;
 			field_names[f].print_fn(&veinfo[idx],
 						field_names[f].index);
 			if (p_outbuffer >= e_buf)
@@ -738,7 +734,7 @@ void print_ve()
 			if (p->next != NULL)
 				*p_outbuffer++ = ' ';
 		}
-		printf("%s\n", g_outbuffer);
+		printf("%s\n", trim_eol_space(g_outbuffer, p_outbuffer));
 		g_outbuffer[0] = 0;
 		p_outbuffer = g_outbuffer;
 	}
@@ -815,14 +811,14 @@ void update_cpu(int veid, unsigned long limit, unsigned long units)
 	return;
 }
 
-void update_la(int veid, struct Cla *la)
+void update_cpustat(int veid, struct Ccpustat *st)
 {
 	struct Cveinfo *tmp;
 
 	if ((tmp = find_ve(veid)) == NULL)
 		return;
-	tmp->la = x_malloc(sizeof(*la));
-	memcpy(tmp->la, la, sizeof(*la));
+	tmp->cpustat = x_malloc(sizeof(*st));
+	memcpy(tmp->cpustat, st, sizeof(*st));
 	return;
 }
 
@@ -855,7 +851,7 @@ void merge_conf(struct Cveinfo *ve, vps_res *res)
 		memset(ve->ubc, 0, sizeof(struct Cubc));
 #define MERGE_UBC(name, ubc, res)				\
 do {								\
-	if (res == NULL || res->ub.name == NULL) 		\
+	if (res == NULL || res->ub.name == NULL)		\
 		break;						\
 	ubc->name[2] = res->ub.name[0];				\
 	ubc->name[3] = res->ub.name[1];				\
@@ -881,6 +877,7 @@ do {								\
 		MERGE_UBC(dcachesize, ve->ubc, res);
 		MERGE_UBC(numfile, ve->ubc, res);
 		MERGE_UBC(numiptent, ve->ubc, res);
+		MERGE_UBC(swappages, ve->ubc, res);
 #undef MERGE_UBC
 	}
 	if (ve->ip == NULL && !list_empty(&res->net.ip)) {
@@ -902,15 +899,17 @@ do {								\
 	{
 		ve->cpu = x_malloc(sizeof(struct Ccpu));
 		memset(ve->cpu, 0, sizeof(struct Ccpu));
-		if (res->cpu.units != NULL)
-			ve->cpu->limit[0] = *res->cpu.units;
 		if (res->cpu.limit != NULL)
-			ve->cpu->limit[1] = *res->cpu.limit;
+			ve->cpu->limit[0] = *res->cpu.limit;
+		if (res->cpu.units != NULL)
+			ve->cpu->limit[1] = *res->cpu.units;
 	}
 	if (res->misc.hostname != NULL)
 		ve->hostname = strdup(res->misc.hostname);
 	if (res->misc.description != NULL)
 		ve->description = strdup(res->misc.description);
+	if (res->tmpl.ostmpl != NULL)
+		ve->ostemplate = strdup(res->tmpl.ostmpl);
 	if (res->name.name != NULL) {
 		int veid_nm = get_veid_by_name(res->name.name);
 		if (veid_nm == ve->veid)
@@ -920,6 +919,12 @@ do {								\
 		ve->ve_root = strdup(res->fs.root);
 	if (res->fs.private != NULL)
 		ve->ve_private = strdup(res->fs.private);
+	ve->onboot = res->misc.onboot;
+	if (res->misc.bootorder != NULL) {
+		ve->bootorder = x_malloc(sizeof(*ve->bootorder));
+		*ve->bootorder = *res->misc.bootorder;
+	}
+	ve->io.ioprio = res->io.ioprio;
 }
 
 int read_ves_param()
@@ -938,6 +943,8 @@ int read_ves_param()
 		ve_root = strdup(param->res.fs.root_orig);
 	if (param->res.fs.private != NULL)
 		ve_private = strdup(param->res.fs.private_orig);
+	if (param->res.cpt.dumpdir != NULL)
+		dumpdir = strdup(param->res.cpt.dumpdir);
 	free_vps_param(param);
 	for (i = 0; i < n_veinfo; i++) {
 		veid = veinfo[i].veid;
@@ -952,10 +959,8 @@ int read_ves_param()
 								ve_private);
 		free_vps_param(param);
 	}
-	if (ve_root != NULL)
-		free(ve_root);
-	if (ve_private != NULL)
-		free(ve_private);
+	free(ve_root);
+	free(ve_private);
 
 	return 0;
 }
@@ -1057,6 +1062,8 @@ int get_ub()
 			limit, failcnt)
 		UPDATE_UBC(name, numiptent, ve.ubc, held, maxheld, barrier, \
 			limit, failcnt)
+		UPDATE_UBC(name, swappages, ve.ubc, held, maxheld, barrier, \
+			limit, failcnt)
 	}
 	if (veid && check_veid_restr(veid)) {
 		update_ubc(veid, ve.ubc);
@@ -1079,7 +1086,7 @@ char *remove_sp(char *str)
 	while (sp < ep) {
 		*tp = *sp;
 		if (isspace(*sp)) {
-			*tp =  ' ';
+			*tp = ' ';
 			skip++;
 		} else
 			skip = 0;
@@ -1094,10 +1101,11 @@ char *remove_sp(char *str)
 char *invert_ip(char *ips)
 {
 	char *tmp, *p, *ep, *tp;
-	int len, rc;
+	size_t len;
+	int rc;
 	unsigned int ip[4];
 	int family;
-	char ip_str[64];
+	char ip_str[INET6_ADDRSTRLEN];
 
 	if (ips == NULL)
 		return NULL;
@@ -1118,8 +1126,8 @@ char *invert_ip(char *ips)
 		strncpy(ip_str, p, len);
 		ip_str[len] = 0;
 		if ((family = get_netaddr(ip_str, ip)) == -1)
-                	continue;;
-	        if ((inet_ntop(family, ip, ip_str, sizeof(ip_str) - 1)) == NULL)
+			continue;;
+		if ((inet_ntop(family, ip, ip_str, sizeof(ip_str) - 1)) == NULL)
 			continue;
 		rc = sprintf(tp, "%s ", ip_str);
 		if (rc == -1)
@@ -1198,7 +1206,7 @@ static inline int get_ve_ips(unsigned int id, char **str)
 		for (i = ret - 1; i >= 0; i--)
 			cp += sprintf(cp, "%d.%d.%d.%d ", NIPQUAD(addr[i]));
 		*cp = '\0';
- 		*str = buf;
+		*str = buf;
 	} else
 		*str = strdup("");
 out:
@@ -1324,7 +1332,7 @@ int get_stop_quota_stat(int veid)
 	while(!feof(fp)) {
 		if (fgets(buf, sizeof(buf), fp) == NULL)
 			break;
-		if (sscanf(buf, "%15s %lu %lu %lu", res,  &usage,
+		if (sscanf(buf, "%15s %lu %lu %lu", res, &usage,
 				&softlimit, &hardlimit) != 4)
 		{
 			continue;
@@ -1356,25 +1364,35 @@ int get_stop_quota_stats()
 	return 0;
 }
 
-int get_ve_la(int veid)
+static long get_clk_tck()
+{
+	if (__clk_tck != -1)
+		return __clk_tck;
+	__clk_tck = sysconf(_SC_CLK_TCK);
+	return __clk_tck;
+}
+
+int get_ve_cpustat(int veid)
 {
 	struct vz_cpu_stat stat;
 	struct vzctl_cpustatctl statctl;
-	struct Cla la;
+	struct Ccpustat st;
 
 	statctl.veid = veid;
 	statctl.cpustat = &stat;
 	if (ioctl(vzctlfd, VZCTL_GET_CPU_STAT, &statctl) != 0)
 		return 1;
-	la.la[0] = stat.avenrun[0].val_int + (stat.avenrun[0].val_frac * 0.01);
-	la.la[1] = stat.avenrun[1].val_int + (stat.avenrun[1].val_frac * 0.01);
-	la.la[2] = stat.avenrun[2].val_int + (stat.avenrun[2].val_frac * 0.01);
+	st.la[0] = stat.avenrun[0].val_int + (stat.avenrun[0].val_frac * 0.01);
+	st.la[1] = stat.avenrun[1].val_int + (stat.avenrun[1].val_frac * 0.01);
+	st.la[2] = stat.avenrun[2].val_int + (stat.avenrun[2].val_frac * 0.01);
 
-	update_la(veid, &la);
+	st.uptime = (float) stat.uptime_jif / get_clk_tck();
+
+	update_cpustat(veid, &st);
 	return 0;
 }
 
-int get_ves_la()
+int get_ves_cpustat()
 {
 	int i;
 
@@ -1383,7 +1401,7 @@ int get_ves_la()
 	for (i = 0; i < n_veinfo; i++) {
 		if (veinfo[i].hide)
 			continue;
-		get_ve_la(veinfo[i].veid);
+		get_ve_cpustat(veinfo[i].veid);
 	}
 	close(vzctlfd);
 	return 0;
@@ -1392,9 +1410,9 @@ int get_ves_la()
 int get_mounted_status()
 {
 	int i;
+	char buf[512];
+
 	for (i = 0; i < n_veinfo; i++) {
-		if (veinfo[i].ve_root == NULL)
-			continue;
 		if (veinfo[i].status == VE_RUNNING)
 			continue;
 		if (veinfo[i].ve_private == NULL ||
@@ -1403,6 +1421,11 @@ int get_mounted_status()
 			veinfo[i].hide = 1;
 			continue;
 		}
+		get_dump_file(veinfo[i].veid, dumpdir, buf, sizeof(buf));
+		if (stat_file(buf))
+			veinfo[i].status = VE_SUSPENDED;
+		if (veinfo[i].ve_root == NULL)
+			continue;
 		if (vps_is_mounted(veinfo[i].ve_root))
 			veinfo[i].status = VE_MOUNTED;
 	}
@@ -1470,11 +1493,11 @@ int get_ve_list()
 
 int search_field(char *name)
 {
-	int i;
+	unsigned int i;
 
 	if (name == NULL)
 		return -1;
-	for (i = 0; i < sizeof(field_names) / sizeof(*field_names); i++) {
+	for (i = 0; i < ARRAY_SIZE(field_names); i++) {
 		if (!strcmp(name, field_names[i].name))
 			return i;
 	}
@@ -1486,7 +1509,8 @@ int build_field_order(char *fields)
 	struct Cfield_order *tmp, *prev = NULL;
 	char *sp, *ep, *p;
 	char name[32];
-	int order, nm_len;
+	int order;
+	size_t nm_len;
 
 	sp = fields;
 	if (fields == NULL)
@@ -1515,7 +1539,6 @@ int build_field_order(char *fields)
 			prev->next = tmp;
 		prev = tmp;
 	} while (sp < ep);
-	last_field = prev;
 	return 0;
 }
 
@@ -1549,11 +1572,11 @@ int collect()
 	}
 	if (check_param(RES_QUOTA))
 		get_run_quota_stat();
-	if (check_param(RES_LA))
-		get_ves_la();
+	if (check_param(RES_CPUSTAT))
+		get_ves_cpustat();
 	if (check_param(RES_CPU))
-		if ((ret = get_ves_cpu()))
-			return 1;
+		if (!only_stopped_ve && (ret = get_ves_cpu()))
+			return ret;
 	read_ves_param();
 	get_mounted_status();
 	if (host_pattern != NULL)
@@ -1567,9 +1590,9 @@ int collect()
 
 void print_names()
 {
-	int i;
+	unsigned int i;
 
-	for (i = 0; i < sizeof(field_names) / sizeof(*field_names); i++)
+	for (i = 0; i < ARRAY_SIZE(field_names); i++)
 		printf("%-15s %-15s\n", field_names[i].name,
 					field_names[i].hdr);
 	return;
@@ -1588,18 +1611,22 @@ void free_veinfo()
 			free(veinfo[i].name);
 		if (veinfo[i].description != NULL)
 			free(veinfo[i].description);
+		if (veinfo[i].ostemplate != NULL)
+			free(veinfo[i].ostemplate);
 		if (veinfo[i].ubc != NULL)
 			free(veinfo[i].ubc);
 		if (veinfo[i].quota != NULL)
 			free(veinfo[i].quota);
-		if (veinfo[i].la != NULL)
-			free(veinfo[i].la);
+		if (veinfo[i].cpustat != NULL)
+			free(veinfo[i].cpustat);
 		if (veinfo[i].cpu != NULL)
 			free(veinfo[i].cpu);
 		if (veinfo[i].ve_root != NULL)
 			free(veinfo[i].ve_root);
 		if (veinfo[i].ve_private != NULL)
 			free(veinfo[i].ve_private);
+		if (veinfo[i].bootorder != NULL)
+			free(veinfo[i].bootorder);
 	}
 }
 
@@ -1684,7 +1711,7 @@ int main(int argc, char **argv)
 	}
 	if (optind < argc) {
 		while (optind < argc) {
-			veid =  strtol(argv[optind], &ep, 10);
+			veid = strtol(argv[optind], &ep, 10);
 			if (*ep != 0 || !veid) {
 				veid = get_veid_by_name(argv[optind]);
 				if (veid < 0) {
@@ -1712,9 +1739,10 @@ int main(int argc, char **argv)
 		return ret;
 	print_ve();
 	free_veinfo();
-	if (host_pattern != NULL) free(host_pattern);
-	if (name_pattern != NULL) free(name_pattern);
-	if (f_order != NULL) free(f_order);
+	free(host_pattern);
+	free(name_pattern);
+	free(desc_pattern);
+	free(f_order);
 
 	return 0;
 }
