@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2009, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@
 #include "util.h"
 #include "logger.h"
 
+extern int page_size;
+
 void usage(int rc)
 {
 	fprintf(rc ? stderr : stdout, "Usage: vzmemcheck [-v] [-A]\n");
@@ -65,17 +67,18 @@ int calculate(int numerator, int verbose)
 	FILE *fd;
 	struct ub_struct ub_s;
 
-	if ((fd = fopen(PROCUBC, "r")) == NULL) {
-		fprintf(stderr, "Unable to open file " PROCUBC ": %s\n",
-			strerror(errno));
-		return 1;
-	}
 	if (get_mem(&mem.ram))
 		return 1;
 	if (get_swap(&mem.swap))
 		mem.swap = 1;
 	if (get_lowmem(&mem.lowmem))
 		return 1;
+
+	if ((fd = fopen(PROCUBC, "r")) == NULL) {
+		fprintf(stderr, "Unable to open file " PROCUBC ": %s\n",
+			strerror(errno));
+		return 1;
+	}
 	mem.lowmem *= 0.4;
 	memset(&ub_s, 0, sizeof(ub_s));
 	memset(&rutotal_comm, 0, sizeof(rutotal_comm));
@@ -201,7 +204,9 @@ int main(int argc, char **argv)
 		}
 	}
 	init_log(NULL, 0, 0, 0, 0, NULL);
-	get_pagesize();
+	page_size = get_pagesize();
+	if (page_size < 0)
+		return 1;
 	ret = calculate(numerator, verbose);
 	exit(ret);
 }
